/*
 * Client Communication Events Manager
 * 1. set your custom event listenter (only 1 handler per event type)
 *      ceeManager.setEventListener('my_event', function(connection, data) {
 *          // connection is [WebServerConnection] made by client
 *          // data is what you put with registerEvent method
 *          return true // the event will be removed from queue (after it's handled)
 *          return false // if you want the event to remain on the list;
 *                       // such event must be registered as persistent
 *      });
 * 2. when the event occurs, register it:
 *      ceeManager.registerEvent('my_event'[, data[, persistent]]);
 *      // calls a handler set for 'my_event' with data attribute
 *      // if persistent is true the event will remain in queue until
 *      // the handler returns true for at least one connection
 *
 * 3. when you've got the connection from client, plug it into manager
 *      ceeManager.plug(connection)
 *
 * NOTICE: If the connection is not closed by handler it will move on the queue
 * and call other handlers for other events
 *
 * NOTICE: If there are many connections pluged and at least ONE will return true
 * the event will be removed from the queue
 */
function CCEManager() {
    
    /*
     * Q: can the same connection be pluged more than once?
     */
    this.plug = function(connection) {
        this.__connections.push(connection);
        this.__dispatchEvent();
        
        this.__watch();
    };
    
    this.__connections = [];
    this.__queue = [];
    this.__events = {};

    //function l() {opera.postError('| ' + arguments);};
    
    this.registerEvent = function(kind, data, persistent) {
        if (!(kind in this.__events)) {
            this.__events[kind] = {
                events : [],
                handler : null,
            }
        }
        this.__events[kind].events.push({
            data : data,
            persistent : !!persistent
        });
        
        this.__queue.push(kind);
        this.__dispatchEvent();
    };
    
    this.setEventListener = function(kind, handler) {
        if (!(kind in this.__events)) {
            this.__events[kind] = {
                events : [],
                handler : null,
            }
        }
        this.__events[kind].handler = typeof handler === 'function' ? handler : false;
    };
    
    this.__removeClosedConnections = function() {
        this.__connections = this.__connections.filter(function(c){return !c.closed;});
    };
    
    this.__dispatchEvent = function(inx) {
        // which event to fire?
        inx = inx || 0;
            
        this.__removeClosedConnections();

        // STOP dispatching (out of the queue)?
        var kind = this.__queue[inx];
        if (!kind) {return;}
        
        var event = this.__events[kind].events[0]; 
        var done = event ? !event.persistent : true;
            
        if (event && !!this.__events[kind].handler) {
            this.__connections.forEach(function(connection){
                // there is a possibility that the connection closed by now...
                if (connection.closed) {return;}    // 2b or !2b
                
                if (this.__events[kind].handler(connection, event.data)) {
                    done = true;
                }
            }, this);
        }
        
        // if done, the event is no longer needed
        if (done) {
            this.__events[kind].events.shift();
            this.__queue.splice(inx--, 1);
        }
        
        // TRY ON
        this.__dispatchEvent(++inx);
    };
    
    
    /*
     * to be changed to addModeChangeListener, for more than 1 handler
     */
     
    this.__modeChangeListener = false;
    
    this.setModeChangeListener = function(handler) {
        this.__modeChangeListener = typeof handler === 'function' ? handler : false;
    };
    
    this.__timeout = 0;
    
    // AWAKE and SLEEPY must be different values (nr of miliseconds)
    // !!ASLEEP must be false
    this.__MODES = {
        AWAKE : 60 * 1000,      // watch requency
        SLEEPY : 2 * 60 * 1000, // wait before falling asleep
        ASLEEP : 0
    };
    
    this.__mode = this.__MODES.AWAKE;
    
    this.__watch = function() {

        if (this.__timeout) {window.clearTimeout(this.__timeout);}
        
        if (!this.__watchdog) {return;}
        
        this.__removeClosedConnections();
        
        if (this.__mode == this.__MODES.ASLEEP) {
            this.__mode = this.__MODES.AWAKE;
            if (!!this.__modeChangeListener) {
                this.__modeChangeListener.call(null, true);
            }
        }
        else {
            this.__mode = this.__connections.length ? this.__MODES.AWAKE : (
                this.__mode == this.__MODES.AWAKE
                ? this.__MODES.SLEEPY : this.__MODES.ASLEEP
            );

            if (
                this.__mode == this.__MODES.ASLEEP 
                && !!this.__modeChangeListener
            ) {
                this.__modeChangeListener.call(null, false);
            }
        }
        
        if (this.__mode) {
            var self = this;
            var func = arguments.callee;
            this.__timeout = window.setTimeout(function() {
                    func.call(self);
                }, this.__mode);
        }
    };
    
    /*
     * any open connections?
     */
    this.isOpen = function() {
        return this.__connections.some(function(conn){return !conn.closed;});
    };
    
    this.__watchdog = false;
    this.__defineSetter__("watchdog", function(watch) {
        this.__watchdog = !!watch;
        this.__watch();
    });
    this.__defineGetter__("watchdog", function() {
        return this.__watchdog;
    });
    
};